@@ -1,7 +1,7 @@
 # PGP::Sign -- Create a PGP signature for data, securely.  -*- perl -*-
-# $Id: Sign.in,v 0.15 2002/06/28 23:27:20 eagle Exp $
+# Sign.in,v 0.17 2004/08/08 09:42:35 eagle Exp
 #
-# Copyright 1997, 1998, 1999, 2000, 2002 Russ Allbery <rra@stanford.edu>
+# Copyright 1997, 1998, 1999, 2000, 2002, 2004 Russ Allbery <rra@stanford.edu>
 #
 # This program is free software; you may redistribute it and/or modify it
 # under the same terms as Perl itself.
@@ -41,7 +41,7 @@ use vars qw(@ERROR @EXPORT @EXPORT_OK @ISA $MUNGE $PGPS $PGPV $PGPPATH
 
 # The current PGP::Sign version number, which is not the same as the CVS
 # revision of this file.  Picked up during build time.
-$VERSION = '0.17';
+$VERSION = '0.19';
 
 ##############################################################################
 # Global variables
@@ -324,7 +324,8 @@ sub pgp_verify {
     # Figure out what command line we'll be using.
     my @command;
     if ($PGPSTYLE eq 'GPG') {
-        @command = ($PGPV, '--batch', '--verify');
+        @command = ($PGPV, '--batch', '--verify', '--quiet' ,'--status-fd=1',
+                    '--logger-fd=1');
     } else {
         @command = ($PGPV, '+batchmode');
     }
@@ -352,8 +353,7 @@ sub pgp_verify {
     # Check for the message that gives us the key status and return the
     # appropriate thing to our caller.  This part is a zoo due to all of the
     # different formats used.  GPG has finally done the right thing and
-    # implemented a separate status stream with parseable data, but I haven't
-    # gotten to implementing that yet....
+    # implemented a separate status stream with parseable data.
     #
     # Partly from pgpverify, by David Lawrence <tale@isc.org>:
     #
@@ -369,23 +369,29 @@ sub pgp_verify {
     #   gpg: Good signature from "Russ Allbery <rra@stanford.edu>"
     #
     # Also, PGP v2 prints out "Bad signature" while PGP v5 uses "BAD
-    # signature", PGP v6 reverts back to "Bad signature", and GPG uses the
-    # same but not at the beginning of a line.
+    # signature" and PGP v6 reverts back to "Bad signature".
     local $_;
     local $/ = '';
     my $signer;
     while (<$output>) {
-        if (/^Good signature from user(?::\s+(.*)|\s+\"(.*)\"\.)$/m) {
-            $signer = $+;
-            last;
-        } elsif (/^Good signature made .* by key:\n.+\n\s+\"(.*)\"/m) {
-            $signer = $1;
-            last;
-        } elsif (/^\S+: Good signature from \"(.*)\"/m) {
-            $signer = $1;
-            last;
-        } elsif (/^(?:\S+: )?Bad signature /im) {
-            last;
+        if ($PGPSTYLE eq 'GPG') {
+            if (/\[GNUPG:\]\s+GOODSIG\s+\S+\s+(.*)/) {
+                $signer = $1;
+                last;
+            }
+        } else {
+            if (/^Good signature from user(?::\s+(.*)|\s+\"(.*)\"\.)$/m) {
+                $signer = $+;
+                last;
+            } elsif (/^Good signature made .* by key:\n.+\n\s+\"(.*)\"/m) {
+                $signer = $1;
+                last;
+            } elsif (/^\S+: Good signature from \"(.*)\"/m) {
+                $signer = $1;
+                last;
+            } elsif (/^(?:\S+: )?Bad signature /im) {
+                last;
+            }
         }
     }
     close $pgp;
@@ -598,9 +604,9 @@ way that I can see to get PGP to verify a detached signature without using
 files on disk, means that in order to maintain the intended default
 semantics of this module (manipulating detached signatures), I had to use
 temporary files in the implementation of pgp_verify().  PGP::Sign sets its
-umask before creating those temporary files and checks for race conditions,
-but files may be left behind in the event that an application using
-pgp_verify() crashes unexpectedly.
+umask before creating those temporary files and opens them with O_EXCL, but
+files may be left behind in the event that an application using pgp_verify()
+crashes unexpectedly.  Setting $PGP::Sign::TMPDIR is recommended.
 
 Also, because of this incompatibility, you need to be aware of what the
 process checking the signatures you generate is expecting.  If that process
@@ -618,6 +624,10 @@ attached and detached signatures, and GnuPG treated them both the same.
 Versions of GnuPG at least after 1.0 appear to have changed to follow the
 PGP behavior instead.
 
+When verification of a signature fails, currently not very much information
+about what failed is available (since an invalid signature isn't considered
+an error in the pgp_error() sense).
+
 =head1 CAVEATS
 
 This module is fairly good at what it does, but it doesn't do very much.  At
@@ -660,25 +670,24 @@ Russ Allbery <rra@stanford.edu>
 
 =head1 COPYRIGHT AND LICENSE
 
-Copyright 1997, 1998, 1999, 2000, 2002 Russ Allbery <rra@stanford.edu>
+Copyright 1997, 1998, 1999, 2000, 2002, 2004 Russ Allbery
+<rra@stanford.edu>.
 
 This program is free software; you may redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =head1 HISTORY
 
-Based heavily on work by Andrew Gierth <andrew@erlenstar.demon.co.uk> and
-benefitting greatly from input, comments, suggestions, and help from him,
-this module came about in the process of implementing PGPMoose signatures
-and control message signatures for Usenet.  PGPMoose is the idea of Greg
-Rose <ggr@usenix.org>, and signcontrol and pgpverify are the idea of David
-Lawrence <tale@isc.org>.
+Based heavily on work by Andrew Gierth and benefitting greatly from input,
+comments, suggestions, and help from him, this module came about in the
+process of implementing PGPMoose signatures and control message signatures
+for Usenet.  PGPMoose is the idea of Greg Rose, and signcontrol and
+pgpverify are the idea of David Lawrence.
 
 Support for PGPPATH, the test suite, some bug fixes, and the impetus to get
-another version released came from Andrew Ford <A.Ford@ford-mason.co.uk>.
-Thank you.
+another version released came from Andrew Ford.  Thank you.
 
-Original support for GnuPG from Todd Underwood <todd@nm.net> and Monte
-Mitzelfelt <monte@nm.net>.
+Original support for GnuPG from Todd Underwood and Monte Mitzelfelt.  Code
+for using --status-fd based on code by Marco d'Itri.
 
 =cut
